# HBACK1 -- Remove background from all spectra in a file, replacing input in Nocint/
# By Doug Mink, SAO Telescope Date Center
# October 24, 2005

procedure hback1 (image)

string	image = ""	{prompt="Input multispec image to correct"}
bool plotspec = yes	{prompt="Plot each spectrum as processed (y or n)"}
bool saveold = yes	{prompt="Save original files in Noback/ (y or n)"}
bool verbose = no	{prompt="Log progress (y or n)"}
bool debug = no		{prompt="Too much info from sumspec (y or n)"}

begin
	bool vb, db, save, pixset, pspec
	file imin, imout, imdisp, imroot, imcal, newfile, templist
	string history
	int lroot, lref, nap

	imin = image
	pspec = plotspec
	vb = verbose
	db = debug
	if (db) {
	    vb = yes
	    }
	save = saveold

	 if (!defpac ("rvsao")) {
	    rvsao (motd=no)
	    }

	if (imin == "") {
	    print ("HBACK1:  Linearize file of spectra")
	    print ("Usage: hlin1 <input image>")
	    return
	    }

# Make output file name and check to make sure input is multispec
	smatch (imin, ".ms")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imroot = substr (imin, 1, lroot)
	    smatch (imin, ".fits")
	    if (!smatch.found) {
		imin = imin // ".fits"
		}
	    }
	else {
	    smatch (imin, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imroot = substr (imin, 1, lroot)
		imin = imroot // ".ms.fits"
		}
	    else {
		imroot = imin
		imin = imroot // ".ms.fits"
		}
	    }
	if (!access (imin)) {
	    print ("HBACK1: ", imin," does not exist")
	    return
	    }

# Do not remove background twice
	imgetp.default = "0"
	imgetp (imin, "HBACK")
	if (imgetp.value != "0") {
	    if (vb) {
		print ("HBACK1: ", imin, " has already had its background removed")
		print ("HBACK = ", imgetp.value)
		}
	    return
	    }

# Make output file name
	imout = imroot // ".noback.ms.fits"

# Get number ofspectra in input file
	imgetp.default = "0"
	imgetp (imin, "i_naxis2")
	nap = int (imgetp.value)

# Fix header paramter for further processing
	imgetp.default = "0"
	imgetp (imin, "CTYPE1")
	pixset = NO
	if (imgetp.value == "PIXEL") {
	    hedit.add = no
	    hedit.addonly = no
	    hedit.delete = no
	    hedit.verify = no
	    hedit.show = vb
	    hedit.update = yes
            hedit (imin, "CTYPE1", "LINEAR")
	    pixset = YES
	    }

# Set parameters for background removal
	contsum.c_interactive = no
	contsum.c_sample = "*"
	contsum.c_function = "spline3"
	contsum.naverage = 1
	contsum.order = 20
	contsum.s_low_reject = 3.
	contsum.s_high_reject = 2.
	contsum.t_low_reject = 2.
	contsum.t_high_reject = 2.
	contsum.niterate = 10
	contsum.grow = 1.

# Rebin spectra identically
	sumspec.spectra = imin
	sumspec.compfile = imout
	sumspec.specnum = "1-" // nap
	sumspec.specband = 0
	sumspec.specdir = ""
	sumspec.compname = ""
	sumspec.compdir = ""
	sumspec.nspec = nap
	sumspec.save_names = no
	sumspec.normin = 0.
	sumspec.fixbad = "no"
	sumspec.cont_remove = "no"
	sumspec.reject = no
	sumspec.abs_reject = 100.
	sumspec.em_reject = 2.
	sumspec.contout = "subtract"
	sumspec.cont_plot = no
	sumspec.cont_add = 0.
	sumspec.spec_smooth = 0
	sumspec.st_lambda = INDEF
	sumspec.end_lambda = INDEF
	sumspec.pix_lambda = INDEF
	sumspec.npts = INDEF
	sumspec.complog = no
	sumspec.interp_mode = "spline3"
	sumspec.normout = 0.
	sumspec.spec_plot = no
	sumspec.spec_int = no
	sumspec.comp_plot = pspec
	sumspec.comp_int = no
	sumspec.velcomp = INDEF
	sumspec.zcomp = INDEF
	sumspec.svel_corr = "none"
	sumspec.nsmooth = 0
	sumspec.device = "stdgraph"
	sumspec.plotter = "stdplot"
	sumspec.logfiles = "STDOUT,background.log"
	sumspec.nsum = 1
	sumspec.debug = db
	sumspec.cursor = ""
	sumspec (imin, imout)

# Fix header paramter for further processing
	if (pixset) {
	    hedit.add = no
	    hedit.addonly = no
	    hedit.delete = no
	    hedit.verify = no
	    hedit.show = vb
	    hedit.update = yes
            hedit (imout, "CTYPE1", "PIXEL")
            hedit (imin, "CTYPE1", "PIXEL")
	    }

# Move original file to Noback directory, if requested
	if (save) {
	    if (!access ("Noback/")) {
		mkdir ("Noback")
		}
	    newfile = "Noback/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    movefiles (imin, "Noback")
	    if (vb) {
		print ("HBACK1: spectrum + continuum ", imin, " => Noback/")
		}
	    }
	else {
	    delete (imin)
	    }

# Rename result file to original file name
	rename (imout, imin)

# Add history line including time
	gettime ()
	history = gettime.date // " " // gettime.time // " continuum removed"
	hedit.add = no
	hedit.addonly = no
	hedit.delete = no
	hedit.verify = no
	hedit.show = vb
	hedit.update = yes
        hedit (imin, "HBACK", history, add+)

end
# Oct 24 2005	New task based on hlin.cl
