# HSPECFIT -- Fit multispec spectra to surrounding spectra and save residuals
# By Doug Mink, SAO Telescope Data Center
# November 27, 2006

procedure hspecfit (images)

string	images = ""		{prompt="Input multispec images to correct"}
string	imdir = ""		{prompt="Input image directory"}
string	apertures="1-300"	{prompt="List of input apertures or columns/lines"}
int	nfit=6			{prompt="Number of surrounding spectra to fit"}
string	outdir="fitspec/"	{prompt="Directory for processed spectrum"}
bool	plotres=no		{prompt="Plot residuals (yes or no)"}
real	fitmax=10.0		{prompt="Maximum fit coefficient"}
bool	deltemp=yes		{prompt="Delete intermediate image files (y or n)"}
bool	verbose = yes		{prompt="Log progress (y or n)"}
bool	debug = no		{prompt="Log progress (y or n)"}

begin
	bool vb, db
	file templist, imin, impath, imout, outpath, imres
	int lroot, ap1, ap2, ap3, ap4, nfit2, nap, naps, ir, iap, nleft
	string history, ims, ap, tchar, imroot, imspec, imline

	ims = images
	vb = verbose
	db = debug
	nfit2 = nfit / 2
	ap = apertures
	if (!access (outdir)) {
	    mkdir (outdir)
	    }
	if (ap == "") {
	    print ("HSPECFIT: You must specify apertures to fit")
	    exit
	    }
	getrange (ap, 0)
	naps = getrange.nval

	if (ims == "") {
	    print ("HSPECFIT:  Fit spectrum to surrounding spectra and save residuals")
	    print ("Usage: hspecfit <input images>")
	    return
	    }

#  Make temporary list of input files
	tchar = substr (ims,1,1)
        if (tchar == '@') {
	    templist = substr (ims,2,100)
	    }
	else {
	    templist = mktemp ("hspecfit") // ".list"
	    files (ims, >templist)
	    }
        list = templist
        while (fscan (list,imin) != EOF) {

# Make sure input is a FITS multispec file if apertures are specified
	    smatch (imin, ".ms")
	    if (!smatch.found) {
		smatch (imin, ".fits")
		if (smatch.found) {
		    lroot = smatch.start - 1
		    imroot = substr (imin, 1, lroot)
		    imin = imroot // ".ms.fits"
		    }
		else {
		    imroot = imin
		    imin = imin // ".ms.fits"
		    }
		if (!access (imin)) {
		    print ("HABSKY: ", imin," does not exist")
		    next
		    }
		}
	    else {
		lroot = smatch.start - 1
		imroot = substr (imin, 1, lroot)
		smatch (imin, ".fits")
		if (!smatch.found) {
		    imin = imin // ".fits"
		    }
		}

	    impath = imdir // imin
	    imres = imroot // "res" // ".ms.fits"
	    copy (impath, imres)

	    imgetp (impath, "i_naxis2")
	    if (imgetp.value != "0") {
		nap = int (imgetp.value)
		}
	    else {
		print ("HSPECFIT: No spectra to fit to")
		next
		}

# Set up parameters to process each object spectrum
	    hspecfit1.objfile = imin
	    hspecfit1.objdir = imdir
	    hspecfit1.fitfile = imin
	    hspecfit1.fitdir = imdir
	    hspecfit1.outfile = imin
	    hspecfit1.outdir = outdir
	    hspecfit1.combine = "average"
	    hspecfit1.reject = "none"
	    hspecfit1.interact = no
	    hspecfit1.plotres = plotres
	    hspecfit1.fitmax = fitmax
	    hspecfit1.deltemp = deltemp
	    hspecfit1.verbose = vb
	    hspecfit1.debug = db

	    for (ir = 1; ir <= naps; ir = ir + 1) {
		getrange (ap, ir)
		iap = getrange.ival
		if (iap < 1) {
		    print ("HSPECFIT: Too few apertures; exit at ", iap)
		    exit
		    }
		hspecfit1.objap = iap
		ap1 = iap - nfit2
		ap2 = iap - 1
		ap3 = iap + 1
		ap4 = iap + nfit2
		if (ap1 < 1) {
		    if (ap2 < 1) {
			ap1 = 2
			ap2 = nfit + 1
			ap3 = 0
			}
		    else {
			ap1 = 1
			nleft = nfit2 - (ap2 - ap1 + 1)
			ap4 = ap4 + nleft
			}
		    }
		else if (ap4 > nap) {
		    if (ap3 > nap) {
			ap1 = nap - nfit
			ap2 = nap - 1
			ap3 = 0
			}
		    else {
			ap4 = nap
			nleft = nfit2 - (ap4 - ap3 + 1)
			ap1 = ap1 - nleft
			}
		    }

		if (ap3 > 0) {
		    hspecfit1.fitaps = ap1 // "-" // ap2 // "," // ap3 // "-" // ap4
		    }
		else {
		    hspecfit1.fitaps = ap1 // "-" // ap2
		    }

#  Make up output name by adding aperture to input file name
		if (iap < 10) {
		    imout = imroot // "_00" // iap
		    }
		else if (iap < 100) {
		    imout = imroot // "_0" // iap
		    }
		else {
		    imout = imroot // "_" // iap
		    }
		imout = imout // ".ms.fits"
		hspecfit1 (imin, imin, imout)
		outpath = outdir // imout
		imspec = outpath // "[*,1,1]"
		imline = imres // "[*," // iap // "]"
		imcopy (imspec, imline)
		if (vb) {
		    print ("HSPECFIT: ", imspec, " -> ", imline)
		    }
		if (deltemp) {
		    delete (outpath)
		    }
		}

# Add history line including time
	    gettime ()
	    history = gettime.date // " " // gettime.time // " " // nfit // " spectra fit"
	    hedit (imres, "HSPECFIT", history, add+, verify-, show-)
	    if (vb) {
		print ("HSPECFIT: ",history)
		}
	    }

	if (tchar != '@' && deltemp) {
	    delete (templist)
	    }
end

# Nov 27 2006	New program
