#*** HSTACK -- Stack single spectrum files into one multispec file
#*** By Doug Mink, SAO
#*** August 31, 2005

procedure hstack (indir, outfile)

string	indir=""	{prompt="Stack all .ms.fits files in this directory"}
string	outfile=""	{prompt="Output multispec file name"}
int	napertures=300	{prompt="Number of apertures in output file"}
int	band=1		{prompt="Band to copy"}
string	oldfile=""	{prompt="File from which to copy keywords"}
bool	deltemp=yes	{prompt="Delete temporary list file (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

	file	keypath, imin, imroot, templist, dirin, imzero, imout, keyfile
	file	filepath, tempfile, imold
	int	lroot, ap, nap
	string	arg, tchar, ims, history, intemp, listarg, filetemp, pathtemp
	string	copykeys, delkeys
	bool	vb

	nap = napertures
	vb = verbose
	dirin = indir
	imout = outfile
	imold = oldfile
	if (!access ("copykeys"))
	    copy (osfn ("hectospec$lib/addstackeys"), "copykeys")
	if (!access ("delkeys"))
	    copy (osfn ("hectospec$lib/delstackeys"), "delkeys")

	if (!defpac ("wcstools")) {
	    wcstools (motd=no)
	    }

	if (!defpac ("rvsao")) {
	    rvsao (motd=no)
	    }

# Make sure that output is a multispec file
	smatch (imout, ".ms.fits")
	if (!smatch.found) {
	    smatch (imout, ".ms")
	    if (smatch.found) {
		imout = imout // ".fits"
		}
	    else {
		smatch (imout, ".fits")
		if (smatch.found) {
		    lroot = smatch.start - 1
		    imroot = substr (imout, 1, lroot)
		    imout = imroot // ".ms.fits"
		    }
		else {
		    imout = imout // ".ms.fits"
		    }
		}
	    }
	if (!access (imout)) {

# Make name for spectrum with all zeroes
	    imzero = mktemp ("zerospec") // ".ms.fits"
	    pathtemp = dirin // "/" // "*.ms.fits"
	    files (pathtemp) | head nlines=1 | scan (imin)
	    sarith.w1 = INDEF
	    sarith.w2 = INDEF
	    sarith.apertures = "1"
	    sarith.bands = "1-3"
	    sarith.beams = ""
	    sarith.apmodulus = 0
	    sarith.reverse = no
	    sarith.ignoreaps = yes
	    sarith.format = "multispec"
	    sarith.renumber = no
	    sarith.offset = 0
	    sarith.clobber = yes
	    sarith.merge = no
	    sarith.rebin = yes
	    sarith.errval = 0.
	    sarith.verbose = vb
	    sarith (imin, "-", imin, imzero, apertures=1, bands="1-3")

	    templist = mktemp ("hstacklist")

# Track down files from input directory
	    for (ap = 1; ap <= nap; ap = ap + 1) {
		tempfile = mktemp ("hstackfile")
		if (ap < 10) {
		    filetemp = "00" // ap // ".*.ms.fits"
		    }
		else if (ap < 100) {
		    filetemp = "0" // ap // ".*.ms.fits"
		    }
		else {
		    filetemp = ap // ".*.ms.fits"
		    }
		pathtemp = dirin // "/" // filetemp
		filepath = ""
		pathnames (pathtemp, > tempfile)
		list = tempfile
		if (fscan (list, filepath) != EOF) {
		    delete (tempfile)
		    filepath = filepath // "[" // band // "]"
		    print (filepath, >> templist)
		    }
		else {
		    delete (tempfile)
		    pathtemp = dirin // "/skydir/" // filetemp
		    pathnames (pathtemp, > tempfile)
		    list = tempfile
		    filepath = ""
		    if (fscan (list, filepath) != EOF) {
			delete (tempfile)
			filepath = filepath // "[" // band // "]"
			print (filepath, >> templist)
			}
		    else {
			delete (tempfile)
			filepath = imzero // "[" // band // "]"
			print (filepath, >> templist)
			}
		    }
		if (vb) {
		    printf ("%03d: %s\n", ap, filepath)
		    }
		}

# Copy multispec sky-subtracted spectra to output multispec file
	    rvsao.sumspec.compfile = imout
	    rvsao.sumspec.specnum = 1
	    rvsao.sumspec.specband = 1
	    rvsao.sumspec.specdir = ""
	    rvsao.sumspec.compname = ""
	    rvsao.sumspec.compdir = ""
	    rvsao.sumspec.nspec = nap
	    rvsao.sumspec.save_names = no
	    rvsao.sumspec.copy_header = yes
	    rvsao.sumspec.normin = 0.
	    rvsao.sumspec.fixbad = "no"
	    rvsao.sumspec.cont_remove = "no"
	    rvsao.sumspec.reject = no
	    rvsao.sumspec.abs_reject = 100.
	    rvsao.sumspec.em_reject = 2.
	    rvsao.sumspec.contout = "no"
	    rvsao.sumspec.cont_plot = no
	    rvsao.sumspec.cont_add = 0.
	    rvsao.sumspec.spec_smooth = 0
	    rvsao.sumspec.st_lambda = INDEF
	    rvsao.sumspec.end_lambda = INDEF
	    rvsao.sumspec.pix_lambda = INDEF
	    rvsao.sumspec.npts = INDEF
	    rvsao.sumspec.complog = no
	    rvsao.sumspec.interp_mode = "spline3"
	    rvsao.sumspec.normout = 0.
	    rvsao.sumspec.spec_plot = no
	    rvsao.sumspec.spec_int = no
	    rvsao.sumspec.comp_plot = vb
	    rvsao.sumspec.comp_int = no
	    rvsao.sumspec.velcomp = INDEF
	    rvsao.sumspec.zcomp = INDEF
	    rvsao.sumspec.svel_corr = "none"
	    rvsao.sumspec.nsmooth = 0
	    rvsao.sumspec.device = "stdgraph"
	    rvsao.sumspec.plotter = "stdplot"
	    rvsao.sumspec.logfiles = "STDOUT,displin.log"
	    rvsao.sumspec.nsum = 1
	    rvsao.sumspec.debug = no
	    rvsao.sumspec.cursor = ""

	    listarg = "@" // templist
	    rvsao.sumspec (listarg, imout)
	    }

# Note that output file already exists and proceed to set up header
	else {
	    printf ("HSTACK: Stacked spectra file %s already exists\n", imout)
	    printf ("HSTACK: Copying header keywords from %s\n", imold)
#	    return
	    }

# Delete fiber-specific keywords
	if (vb) {
	    arg = "-v " // imout // " @delkeys"
	    }
	else {
	    arg = imout // " @delkeys"
	    }
	dhead (arg)
	if (deltemp) {
	    delete ("delkeys")
	    }

# Copy template multispec file keywords to output multispec file
	if (vb) {
	    arg = "-v " // imold // " " // imout // " @copykeys"
	    }
	else {
	    arg = imold // " " // imout // " @copykeys"
	    }
	chead (arg)
	if (deltemp) {
	    delete ("copykeys")
	    }

# Add history line including time
	gettime ()
	history = gettime.date // " " // gettime.time // " images stacked from " // dirin
	if (vb) {
	    hedit (imout, "HSTACK", history, add+, verify-, show+)
	    }
	else {
	    hedit (imout, "HSTACK", history, add+, verify-, show-)
	    }

	if (deltemp) {
	    delete (templist)
	    delete (imzero)
	    }
end
# Aug 31 2005	New task
