# RESUB - Subtract rescaled sky from object+skies and
#	  optionally run XCSAO on new object spectra
# March 3, 2006
# By Doug Mink, SAO Telescope Data Center

procedure resub (object)

file	object=""	{prompt="Object multispec file or @list"}
file	objdir=""	{prompt="Directory containing spectra"}
file	outdir=""	{prompt="Directory for output spectra"}
file	normkey=""	{prompt="Keyword with sky normalization"}
bool	newsky=yes	{prompt="Resubtract sky spectrum (y or n)"}
bool	skyfit=no	{prompt="Fit sky to object+sky and subtract (y or n)"}
bool	crosscor=no	{prompt="Cross-correlate new spectrum (y or n)"}
bool	deltemp=yes	{prompt="Delete temporary list file (y or n)"}
bool	verbose=no	{prompt="Log progress (yes or no)"}
bool	debug=no	{prompt="Print debugging information (y or n)"}

begin
	file ims, imin, templist, fname, tempobj, tempsky, impath, tempobjsky
	char tchar
	string specobj, specsky, specobjsky, inspec, cstring
	file odir, idir, temppath
	real fnorm

	real constant
	real nd
	int im
	bool intact
	bool xcor
	char xmode

	if (crosscor)
	    rvsao (motd=no)
	if (!skyfit)
	    wcstools (motd=no)

        ims = object
	idir = objdir
	odir = outdir
	xcor = crosscor

	tempobj = mktemp ("resubobj") // ".fits"
	tempsky = mktemp ("resubsky") // ".fits"
	tempobjsky = mktemp ("resubobjsky") // ".fits"
	if (skyfit) {
	    hskyfit1.objap = 2
	    hskyfit1.objdir = objdir
	    hskyfit1.skyap = 3
	    hskyfit1.skydir = objdir
	    hskyfit1.outdir = outdir
	    hskyfit1.verbose = verbose
	    hskyfit1.debug = debug
	    }

        tchar = substr (ims,1,1)
        if (tchar == '@') {
            templist = substr (ims,2,100)
            }
        else {
            templist = mktemp ("resublist")
            files (ims, >templist)
            }
	print ("Input file list is ",templist)

        list = templist
        while (fscan (list,imin) != EOF) {
	    impath = idir // imin
	    if (newsky) {
		specsky = imin // "[*,*,3]"
		specobjsky = imin // "[*,*,2]"
		specobj = imin
		if (skyfit) {
		    hskyfit1 (imin,imin,specobj)
		    }
		else if (normkey != "") {
		    fnorm = real (imgetp.value)
		    imgetp (impath, "APRATIO")
		    imarith (specsky, "*", fnorm, tempsky)
		    imarith (specobjsky, "-", tempsky, tempobj)
		    if (verbose) {
			print (impath," background * ",fnorm," subtracted")
			}
		    }
		else {
		    imarith (specobjsky, "-", specsky, tempobj)
		    fnorm = 1.0
		    hedit (tempobj, "SKYRATIO", fnorm, update+)
		    if (verbose) {
			print (impath," background subtracted")
			}
		    }
		if (!skyfit) {
		    imcopy (specsky, tempsky)
		    imcopy (specobjsky, tempobjsky)
		    if (debug) {
			imhead (tempobj)
			imhead (tempobjsky)
			imhead (tempsky)
			}
		    arg = "-o " // specobj // " " // tempobj // " " // tempobjsky
		    arg = arg // " " // tempsky
		    if (debug) {
			print ("imstack ",arg)
			}
		    istack (arg)
		    if (!skyfit) {
			hedit (specobj, "SKYRATIO", fnorm, update+)
			}
		    }
		}
	    else {
		fname = imin
		}
	    if (!skyfit) {
		delete (tempsky)
		delete (tempobj)
		delete (tempobjsky)
		}

# Cross-correlate spectrum against template, producing output in xcout
	    if (xcor) {
		if (newsky) {
		    xcsao.specdir = odir
		    }
		else {
		    xcsao.specdir = idir
		    }
		xcsao (fname,specnum=1,logfiles="xcsao.log,STDOUT",debug=debug)
		if (newsky && deltemp) {
		    delete (fname)
		    }
		}
	    }

# Delete temporary list file
	if (deltemp && tchar != '@') {
	    delete (templist)
	    }
end

# Nov 28 2005	New task
# Dec  1 2005	Make cross-correlation optional
# Dec 16 2005	Add option to normalize sky using file header keyword
# Dec 20 2005	Add option to fit and subtract sky using svdres

# Jan  4 2006	Update svdres parameters
# Jan  5 2006	Add verbose parameter to log progress
# Mar  6 2006	Update to write multispec file in svdres using hskyfit1
