# File t_haphead.x
# By Doug Mink
# December 19, 2006

include	<mach.h>
include	<imhdr.h>
include <imio.h>
include	<math.h>
include <imset.h>


# T_HAPHEAD -- Set APID information plus fiber area in scopy'ed spectrum

procedure t_haphead ()

pointer	im
pointer	immap(), list
char	imname[SZ_FNAME]
int	imtopenp(), imtgetim()
bool	clgetb()
errchk	immap()
bool	verbose, debug

begin

	# Open the list of input images
	list = imtopenp ("images")
	verbose = clgetb ("verbose")
	debug = clgetb ("debug")

	# Loop through the input images.
	im = NULL
	while (imtgetim (list, imname, SZ_FNAME) != EOF) {

	    # Open the image.
	    iferr (im = immap (imname, READ_WRITE, 0)) {
		call printf ("Error reading image %s ...\n")
		    call pargstr (imname)
		next
		}
	    if (im == NULL)
		next

	    # Set fiber area as FIBAREA in spectrum header
	    IM_UPDATE(im) = NO
	    
	    call hap_setarea (im, imname, verbose, debug)

	    # Unmap the image
	    call imunmap (im)
	    im = NULL
	    }

	call imtclose (list)
end


# HAP_SETHEAD -- Set keywords for this aperture in the spectrum header

procedure hap_setarea (im, imname, verbose, debug)

pointer	im		# Image data structure
char	imname[ARB]	# Image file name
bool	verbose		# If true, print table of areas
bool	debug		# If true, print table of positions

double	ra, dindef, dec, pra, pdec
double	angle
double	area
double	x, y, epoch
int	numcat, fiber, aperture, type
char	apidstr[SZ_LINE], apnumstr[SZ_LINE], catnum[16]
char	rastr[16], decstr[16], prastr[16], pdecstr[16], object[32]
char	x1str[16], x2str[16], xrange[32]
char	newname[SZ_FNAME]
char	badchars[SZ_LINE]
char	field[SZ_FNAME]	# Name of pointing
char	colon
int	lobj, i
int	stridx(), strlen(), strcmp()

begin
	dindef = INDEFD
	colon = ':'
	call strcpy (",!;&()[]{}@|\\#$%:<>?*\"\'", badchars, SZ_LINE)

# Read pointing name from APERTURE keyword
	call imgspar (im, "APERTURE", field, SZ_FNAME)

# Read APNUMn as APNUM1 information in header
	call imgspar (im, "APNUM1", apnumstr, SZ_LINE)
	aperture = 0
	type = 0
	x1str[1] = EOS
	x2str[1] = EOS
	call sscan (apnumstr)
	    call gargi (aperture)
	    call gargi (type)
	    call gargwrd (x1str, 16)
	    call gargwrd (x2str, 16)
	if (aperture < 1 || strlen (x1str) < 1) {
	    call printf ("*** No information in APNUM1\n");
	    return
	    }
	call strcpy (x1str, xrange, 32)
	call strcat (":", xrange, 32)
	call strcat (x2str, xrange, 32)

# Read former APIDn information from OBJECT in header (i_title to IRAF)
	call imgspar (im, "i_title", apidstr, SZ_LINE)
	rastr[1] = EOS
	decstr[1] = EOS
	object[1] = EOS
	call sscan (apidstr)
	    call gargwrd (object, 32)
	    call gargwrd (rastr, 16)
	    call gargwrd (decstr, 16)
	    call gargwrd (catnum, 16)
	    call gargi (fiber)
	    call gargd (x)
	    call gargd (y)
	epoch = -1.0
	call imgdpar (im, "EPOCH", epoch)
	if (epoch < 0.0)
	    epoch = 2000.0

# Set name of object if it's generic
	call imastr (im, "CATOBJ", object)
	if (strcmp (object, "target") == 0) {
	    call strcpy (field, object, 32)
	    call strcat ("_", object, 32)
	    call strcat (catnum, object, 32)
	    }
	call sscan (catnum)
	    call gargi (numcat)

# Direction of observed object (Convert RA to degrees if sexigesimal)
	if (strlen (rastr) < 1 || strlen (decstr) < 1) {
	    call printf ("*** No RA or DEC for object in image header\n");
	    return
	    }
	call sscan (rastr)
	    call gargd (ra)
	if (stridx (colon, rastr) != 0)
	    ra = ra * 15.0
	call sscan (decstr)
	    call gargd (dec)

# Direction of plate center (Convert RA to degrees if sexigesimal)
	prastr[1] = EOS
	call imgspar (im, "RA", prastr, SZ_LINE)
	if (strlen (prastr) < 1)
	    call imgspar (im, "CAT-RA", prastr, SZ_LINE)
	if (strlen (prastr) > 0) {
	    call sscan (prastr)
		call gargd (pra)
	    if (stridx (colon, prastr) != 0)
		pra = pra * 15.0
	    }
	pdecstr[1] = EOS
	call imgspar (im, "DEC", pdecstr, SZ_LINE)
	if (strlen (pdecstr) < 1)
	    call imgspar (im, "CAT-DEC", pdecstr, SZ_LINE)
	if (strlen (pdecstr) > 0) {
	    call sscan (pdecstr)
		call gargd (pdec)
	    }

	if (strlen (prastr) < 1 || strlen (pdecstr) < 1) {
	    call printf ("*** No pointing center in image header\n");
	    return
	    }

	if (debug) {
            call printf ("%s: axis: %s %s fiber:%s %s\n")
		call pargstr (imname)
		call pargstr (prastr)
		call pargstr (pdecstr)
		call pargstr (rastr)
		call pargstr (decstr)
	    }

	IM_UPDATE(im) = YES
	call imastr (im, "APID1", apidstr)
	call imastr (im, "i_title", object)
	call imaddi (im, "TYPE", type)
	call imastr (im, "POINTING", field)
	call imaddi (im, "CATNUM", numcat)
	call imastr (im, "RA", rastr)
	call imastr (im, "DEC", decstr)
	call imastr (im, "PRA", prastr)
	call imastr (im, "PDEC", pdecstr)
	call imaddd (im, "EPOCH", epoch)
	call imaddi (im, "FIBER", fiber)
	call imaddd (im, "FIBER_X", x)
	call imaddd (im, "FIBER_Y", y)
	call imaddi (im, "APERTURE", aperture)
	call imastr (im, "APRANGE", xrange)
	call imaddi (im, "DISPAXIS", 1)

# Clean up object name for IRAF
	lobj = strlen (object)
	for (i = 1; i <= lobj; i = i + 1) {
	    if (stridx (object[i], badchars) > 0)
		object[i] = '_'
	    if (object[i] == '-')
		object[i] = 'm'
	    }
	i = lobj
	while (object[i] == '_') {
	    object[i] = EOS
	    i = i - 1
	    }

# Combine keyword values to make a new file name
	call sprintf (newname, SZ_FNAME, "%03d.%s.fits")
	    call pargi (aperture)
	    call pargstr (object)
	call clpstr ("newname", newname)

# Compute angle from optical axis (pointing direction) to fiber
	call hap_diff (pra, pdec, ra, dec, debug, angle)

# Compute area of fiber in square arcseconds
	call hap_comparea (imname, debug, angle, area)

# Write area subtended by the fiber to the spectrum header
	call imaddd (im,"FIBAREA",area)
	return
end


# HAP_COMPAREA -- Compute fiber area from angle from optical axis

procedure hap_comparea (imname, debug, angle, area)

char	imname[ARB]	# Name of spectrum file
bool	debug		# If true, print table of results
double  angle		# Angle on sky from optical axis in degrees
double  area		# Area of sky in this fiber in square arcseconds

double	r, radial, tangential
double	deg, deg2, deg3, deg4, deg5

begin
	deg = angle
	deg2 = deg * deg
	deg3 = deg2 * deg
	deg4 = deg3 * deg
	deg5 = deg4 * deg

	r = 599.9472d0 * deg + 37.369d0 * deg3 + 28.0281d0 * deg5

	radial = (599.9472d0 + 3.d0*37.369d0*deg2 + 5.d0*28.0281d0*deg4) / 3600.d0;

	if ( deg == 0 )
            tangential = radial
	else
            tangential = r / deg / 3600.d0

	area = 3.1415926535d0 / radial / tangential * 0.125d0 * 0.125d0;

	if (debug) {
            call printf ("angle=%7.3f r=%7.3f rad=%8.6f tan=%8.6f area=%8.6f\n")
		call pargd (deg)
		call pargd (r)
		call pargd (radial)
		call pargd (tangential)
		call pargd (area)
	    }
end


# HAP_DIFF -- Compute distance in degrees between two sky coordinates

procedure hap_diff (ra1,dec1,ra2,dec2, debug, angle)

double ra1, dec1	# Right ascension and declination in degrees
double ra2, dec2	# Right ascension and declination in degrees
bool debug		# True for debugging information
double angle		# Separation angle in degrees (returned)

double xr1, xr2, yr1, yr2, pi, deg2rad, rad2deg
double pos1[3], pos2[3], w, diff, cosb
int i

begin
	pi =  3.141592653589793238462643d0
	deg2rad = pi / 180.d0
	rad2deg = 180.d0 / pi

#  Convert two vectors to direction cosines
	xr1 = ra1 * deg2rad
	yr1 = dec1 * deg2rad
	cosb = cos (yr1)
	pos1[0] = cos (xr1) * cosb
	pos1[1] = sin (xr1) * cosb
	pos1[2] = sin (yr1)

	xr2 = ra2 * deg2rad
	yr2 = dec2 * deg2rad
	cosb = cos (yr2)
	pos2[0] = cos (xr2) * cosb
	pos2[1] = sin (xr2) * cosb
	pos2[2] = sin (yr2)

# Modulus squared of half the difference vector
	w = 0.0
	for (i = 0; i < 3; i = i + 1) {
	    w = w + (pos1[i] - pos2[i]) * (pos1[i] - pos2[i])
	    }
	w = w / 4.0
	if (w > 1.0) w = 1.0

# Angle beween the vectors
	diff = 2.0 * atan2 (sqrt (w), sqrt (1.0 - w))
	angle = diff * rad2deg

	if (debug) {
	    call printf ("axis: %9.5f %9.5f fiber: %9.5f %9.5f angle= %9.5f\n")
		call pargd (ra1)
		call pargd (dec1)
		call pargd (ra2)
		call pargd (dec2)
		call pargd (angle)
	    }
	return

end
# Dec 19 2006	New program
